﻿using System;


namespace WeakEventPattern
{
    class Program
    {
        static void TriggerGC()
        {
            Console.WriteLine("Starting GC.");

            GC.Collect();
            GC.WaitForPendingFinalizers();
            GC.Collect();

            Console.WriteLine("GC finished.");
        }

        static void TestNaiveEventListener()
        {
            Console.WriteLine("=== Naive listener (bad) ===");

            EventSource source = new EventSource();

            NaiveEventListener listener = new NaiveEventListener(source);

            source.Raise();

            Console.WriteLine("Setting listener to null.");
            listener = null;

            TriggerGC();

            source.Raise();

            Console.WriteLine("Setting source to null.");
            source = null;

            TriggerGC();
        }

        static void TestLecacyWeakEventListener()
        {
            Console.WriteLine("=== Legacy weak listener (better) ===");

            EventSource source = new EventSource();

            LegacyWeakEventListener listener = new LegacyWeakEventListener(source);

            source.Raise();

            Console.WriteLine("Setting listener to null.");
            listener = null;

            TriggerGC();

            source.Raise();

            Console.WriteLine("Setting source to null.");
            source = null;

            TriggerGC();
        }

        static void TestDotNet45WeakEventListener()
        {
            Console.WriteLine("=== .Net 4.5 weak listener (best) ===");

            EventSource source = new EventSource();

            WeakEventListener listener = new WeakEventListener(source);

            source.Raise();

            Console.WriteLine("Setting listener to null.");
            listener = null;

            TriggerGC();

            source.Raise();

            Console.WriteLine("Setting source to null.");
            source = null;

            TriggerGC();
        }

        static void Main(string[] args)
        {
            TestNaiveEventListener();
            TestLecacyWeakEventListener();
            TestDotNet45WeakEventListener();

            Console.ReadLine();
        }
    }
}
